/**
 * @file artis/kernel/States.hpp
 * @author See the AUTHORS file
 */

/*
 * Copyright (C) 2012-2016 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARTIS_KERNEL_STATES_HPP
#define __ARTIS_KERNEL_STATES_HPP

#include <artis/kernel/State.hpp>
#include <vector>

namespace artis { namespace kernel {

template < typename T, typename U >
class States
{
public:
    States()
    { }

    virtual ~States()
    { }

    virtual void restore(const State < U >& state)
    {
        unsigned int index = 0;

        for (typename std::vector < bool T::* >::iterator it =
                 statesB.begin(); it != statesB.end(); ++it) {
            state.get_state(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < int T::* >::iterator it =
                 statesI.begin(); it != statesI.end(); ++it) {
            state.get_state(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < double T::* >::iterator it =
                 statesD.begin(); it != statesD.end(); ++it) {
            state.get_state(index, *it);
            ++index;
        }
    }

    virtual void save(State < U >& state) const
    {
        unsigned int index = 0;

        for (typename std::vector < bool T::* >::const_iterator it =
                 statesB.begin(); it != statesB.end(); ++it) {
            state.add_state(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < int T::* >::const_iterator it =
                 statesI.begin(); it != statesI.end(); ++it) {
            state.add_state(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < double T::* >::const_iterator it =
                 statesD.begin(); it != statesD.end(); ++it) {
            state.add_state(index, *it);
            ++index;
        }
    }

    void stateB(unsigned int index, bool T::* var)
    {
        if (statesB.size() <= index) {
            statesB.resize(index + 1);
        }
        statesB[index] = var;
    }

    void stateI(unsigned int index, int T::* var)
    {
        if (statesI.size() <= index) {
            statesI.resize(index + 1);
        }
        statesI[index] = var;
    }

    void state(unsigned int index, double T::* var)
    {
        if (statesD.size() <= index) {
            statesD.resize(index + 1);
        }
        statesD[index] = var;
    }

protected:
    std::vector < bool T::* > statesB;
    std::vector < int T::* > statesI;
    std::vector < double T::* > statesD;
};

} }

#endif
